@@ -16,23 +16,23 @@
 # Requires Python 2.6+ and Openssl 1.0+
 #
 
-import os
-import re
-import pwd
-import shutil
-import socket
-import array
-import struct
-import fcntl
-import time
-import base64
+import os  # pylint: disable=W0611
+import re  # pylint: disable=W0611
+import pwd  # pylint: disable=W0611
+import shutil  # pylint: disable=W0611
+import socket  # pylint: disable=W0611
+import array  # pylint: disable=W0611
+import struct  # pylint: disable=W0611
+import fcntl  # pylint: disable=W0611
+import time  # pylint: disable=W0611
+import base64  # pylint: disable=W0611
 import azurelinuxagent.common.conf as conf
 import azurelinuxagent.common.logger as logger
-from azurelinuxagent.common.future import ustr, bytebuffer
+from azurelinuxagent.common.future import ustr, bytebuffer  # pylint: disable=W0611
 from azurelinuxagent.common.exception import OSUtilError, CryptError
 import azurelinuxagent.common.utils.fileutil as fileutil
 import azurelinuxagent.common.utils.shellutil as shellutil
-import azurelinuxagent.common.utils.textutil as textutil
+import azurelinuxagent.common.utils.textutil as textutil  # pylint: disable=W0611
 from azurelinuxagent.common.utils.cryptutil import CryptUtil
 from azurelinuxagent.common.osutil.default import DefaultOSUtil
 
@@ -50,16 +50,16 @@ class Redhat6xOSUtil(DefaultOSUtil):
         return shellutil.run("/sbin/service sshd condrestart", chk_err=False)
 
     def stop_agent_service(self):
-        return shellutil.run("/sbin/service waagent stop", chk_err=False)
+        return shellutil.run("/sbin/service {0} stop".format(self.service_name), chk_err=False)
 
     def start_agent_service(self):
-        return shellutil.run("/sbin/service waagent start", chk_err=False)
+        return shellutil.run("/sbin/service {0} start".format(self.service_name), chk_err=False)
 
     def register_agent_service(self):
-        return shellutil.run("chkconfig --add waagent", chk_err=False)
+        return shellutil.run("chkconfig --add {0}".format(self.service_name), chk_err=False)
 
     def unregister_agent_service(self):
-        return shellutil.run("chkconfig --del waagent", chk_err=False)
+        return shellutil.run("chkconfig --del {0}".format(self.service_name), chk_err=False)
 
     def openssl_to_openssh(self, input_file, output_file):
         pubkey = fileutil.read_file(input_file)
@@ -68,12 +68,11 @@ class Redhat6xOSUtil(DefaultOSUtil):
             ssh_rsa_pubkey = cryptutil.asn1_to_ssh(pubkey)
         except CryptError as e:
             raise OSUtilError(ustr(e))
-        fileutil.write_file(output_file, ssh_rsa_pubkey)
+        fileutil.append_file(output_file, ssh_rsa_pubkey)
 
     # Override
     def get_dhcp_pid(self):
-        ret = shellutil.run_get_output("pidof dhclient", chk_err=False)
-        return ret[1] if ret[0] == 0 else None
+        return self._get_dhcp_pid(["pidof", "dhclient"])
 
     def set_hostname(self, hostname):
         """
@@ -82,7 +81,7 @@ class Redhat6xOSUtil(DefaultOSUtil):
         fileutil.update_conf_file('/etc/sysconfig/network',
                                   'HOSTNAME',
                                   'HOSTNAME={0}'.format(hostname))
-        shellutil.run("hostname {0}".format(hostname), chk_err=False)
+        self._run_command_without_raising(["hostname", hostname], log_error=False)
 
     def set_dhcp_hostname(self, hostname):
         ifname = self.get_if_name()
@@ -98,6 +97,11 @@ class Redhat6xOSUtil(DefaultOSUtil):
 class RedhatOSUtil(Redhat6xOSUtil):
     def __init__(self):
         super(RedhatOSUtil, self).__init__()
+        self.service_name = self.get_service_name()
+
+    @staticmethod
+    def get_systemd_unit_file_install_path():
+        return "/usr/lib/systemd/system"
 
     def set_hostname(self, hostname):
         """
@@ -105,9 +109,12 @@ class RedhatOSUtil(Redhat6xOSUtil):
         Due to a bug in systemd in Centos-7.0, if this call fails, fallback
         to hostname.
         """
-        hostnamectl_cmd = "hostnamectl set-hostname {0} --static".format(hostname)
-        if shellutil.run(hostnamectl_cmd, chk_err=False) != 0:
-            logger.warn("[{0}] failed, attempting fallback".format(hostnamectl_cmd))
+        hostnamectl_cmd = ['hostnamectl', 'set-hostname', hostname, '--static']
+
+        try:
+            shellutil.run_command(hostnamectl_cmd, log_error=False)
+        except shellutil.CommandError:
+            logger.warn("[{0}] failed, attempting fallback".format(' '.join(hostnamectl_cmd)))
             DefaultOSUtil.set_hostname(self, hostname)
 
     def publish_hostname(self, hostname):
@@ -118,10 +125,10 @@ class RedhatOSUtil(Redhat6xOSUtil):
         super(RedhatOSUtil, self).publish_hostname(hostname)
 
     def register_agent_service(self):
-        return shellutil.run("systemctl enable waagent", chk_err=False)
+        return shellutil.run("systemctl enable {0}".format(self.service_name), chk_err=False)
 
     def unregister_agent_service(self):
-        return shellutil.run("systemctl disable waagent", chk_err=False)
+        return shellutil.run("systemctl disable {0}".format(self.service_name), chk_err=False)
 
     def openssl_to_openssh(self, input_file, output_file):
         DefaultOSUtil.openssl_to_openssh(self, input_file, output_file)
@@ -135,3 +142,25 @@ class RedhatOSUtil(Redhat6xOSUtil):
             endpoint = self.get_endpoint_from_leases_path('/var/lib/NetworkManager/dhclient-*.lease')
 
         return endpoint
+
+
+class RedhatOSModernUtil(RedhatOSUtil):
+    def __init__(self):  # pylint: disable=W0235
+        super(RedhatOSModernUtil, self).__init__()
+
+    def restart_if(self, ifname, retries=3, wait=5):
+        """
+        Restart an interface by bouncing the link. systemd-networkd observes
+        this event, and forces a renew of DHCP.
+        """
+        retry_limit = retries + 1
+        for attempt in range(1, retry_limit):
+            return_code = shellutil.run("ip link set {0} down && ip link set {0} up".format(ifname))
+            if return_code == 0:
+                return
+            logger.warn("failed to restart {0}: return code {1}".format(ifname, return_code))
+            if attempt < retry_limit:
+                logger.info("retrying in {0} seconds".format(wait))
+                time.sleep(wait)
+            else:
+                logger.warn("exceeded restart retries")
