@@ -20,19 +20,18 @@
 import os
 import sys
 import time
-import traceback
 
 import azurelinuxagent.common.conf as conf
 import azurelinuxagent.common.logger as logger
 import azurelinuxagent.common.utils.fileutil as fileutil
 
-from azurelinuxagent.common.cgroups import CGroups
-from azurelinuxagent.common.event import add_event, WALAEventOperation
+from azurelinuxagent.common.event import add_event, WALAEventOperation, initialize_event_logger_vminfo_common_parameters
 from azurelinuxagent.common.future import ustr
 from azurelinuxagent.common.osutil import get_osutil
-from azurelinuxagent.common.protocol import get_protocol_util
-from azurelinuxagent.common.protocol.wire import WireClient
+from azurelinuxagent.common.protocol.goal_state import GoalState, GoalStateProperties
+from azurelinuxagent.common.protocol.util import get_protocol_util
 from azurelinuxagent.common.rdma import setup_rdma_device
+from azurelinuxagent.common.utils import textutil
 from azurelinuxagent.common.version import AGENT_NAME, AGENT_LONG_NAME, \
     AGENT_VERSION, \
     DISTRO_NAME, DISTRO_VERSION, PY_VERSION_MAJOR, PY_VERSION_MINOR, \
@@ -60,16 +59,19 @@ class DaemonHandler(object):
         self.osutil = get_osutil()
 
     def run(self, child_args=None):
-        logger.info("{0} Version:{1}", AGENT_LONG_NAME, AGENT_VERSION)
+        #
+        # The Container ID in telemetry events is retrieved from the goal state. We can fetch the goal state
+        # only after protocol detection, which is done during provisioning.
+        #
+        # Be aware that telemetry events emitted before that will not include the Container ID.
+        #
+        logger.info("{0} Version: {1}", AGENT_LONG_NAME, AGENT_VERSION)
         logger.info("OS: {0} {1}", DISTRO_NAME, DISTRO_VERSION)
-        logger.info("Python: {0}.{1}.{2}", PY_VERSION_MAJOR, PY_VERSION_MINOR,
-                    PY_VERSION_MICRO)
+        logger.info("Python: {0}.{1}.{2}", PY_VERSION_MAJOR, PY_VERSION_MINOR, PY_VERSION_MICRO)
 
         self.check_pid()
         self.initialize_environment()
 
-        CGroups.setup()
-
         # If FIPS is enabled, set the OpenSSL environment variable
         # Note:
         # -- Subprocesses inherit the current environment
@@ -79,8 +81,8 @@ class DaemonHandler(object):
         while self.running:
             try:
                 self.daemon(child_args)
-            except Exception as e:
-                err_msg = traceback.format_exc()
+            except Exception as e:  # pylint: disable=W0612
+                err_msg = textutil.format_exception(e)
                 add_event(name=AGENT_NAME, is_success=False, message=ustr(err_msg),
                           op=WALAEventOperation.UnhandledError)
                 logger.warn("Daemon ended with exception -- Sleep 15 seconds and restart daemon")
@@ -116,15 +118,19 @@ class DaemonHandler(object):
             fileutil.mkdir(conf.get_lib_dir(), mode=0o700)
             os.chdir(conf.get_lib_dir())
 
+    def _initialize_telemetry(self):
+        protocol = self.protocol_util.get_protocol()
+        initialize_event_logger_vminfo_common_parameters(protocol)
+
     def daemon(self, child_args=None):
         logger.info("Run daemon")
 
-        self.protocol_util = get_protocol_util()
-        self.scvmm_handler = get_scvmm_handler()
-        self.resourcedisk_handler = get_resourcedisk_handler()
-        self.rdma_handler = get_rdma_handler()
-        self.provision_handler = get_provision_handler()
-        self.update_handler = get_update_handler()
+        self.protocol_util = get_protocol_util()  # pylint: disable=W0201
+        self.scvmm_handler = get_scvmm_handler()  # pylint: disable=W0201
+        self.resourcedisk_handler = get_resourcedisk_handler()  # pylint: disable=W0201
+        self.rdma_handler = get_rdma_handler()  # pylint: disable=W0201
+        self.provision_handler = get_provision_handler()  # pylint: disable=W0201
+        self.update_handler = get_update_handler()  # pylint: disable=W0201
 
         if conf.get_detect_scvmm_env():
             self.scvmm_handler.run()
@@ -138,9 +144,14 @@ class DaemonHandler(object):
 
         self.provision_handler.run()
 
+        # Once we have the protocol, complete initialization of the telemetry fields
+        # that require the goal state and IMDS
+        self._initialize_telemetry()
+
         # Enable RDMA, continue in errors
         if conf.enable_rdma():
-            self.rdma_handler.install_driver()
+            nd_version = self.rdma_handler.get_rdma_version()
+            self.rdma_handler.install_driver_if_needed()
 
             logger.info("RDMA capabilities are enabled in configuration")
             try:
@@ -149,12 +160,10 @@ class DaemonHandler(object):
                 #   incarnation number. A forced update ensures the most
                 #   current values.
                 protocol = self.protocol_util.get_protocol()
-                client = protocol.client
-                if client is None or type(client) is not WireClient:
-                    raise Exception("Attempt to setup RDMA without Wireserver")
-                client.update_goal_state(forced=True)
 
-                setup_rdma_device()
+                goal_state = GoalState(protocol, goal_state_properties=GoalStateProperties.SharedConfig)
+
+                setup_rdma_device(nd_version, goal_state.shared_conf)
             except Exception as e:
                 logger.error("Error setting up rdma device: %s" % e)
         else:
@@ -162,5 +171,10 @@ class DaemonHandler(object):
 
         self.sleep_if_disabled()
 
+        # Disable output to /dev/console once provisioning has completed
+        if logger.console_output_enabled():
+            logger.info("End of log to /dev/console. The agent will now check for updates and then will process extensions.")
+            logger.disable_console_output()
+
         while self.running:
             self.update_handler.run_latest(child_args=child_args)
