@@ -15,19 +15,16 @@
 # Requires Python 2.6+ and Openssl 1.0+
 #
 
-from azurelinuxagent.common.version import DISTRO_NAME, DISTRO_VERSION, \
-                                     DISTRO_FULL_NAME
+from distutils.version import LooseVersion as Version  # pylint: disable=no-name-in-module, import-error
 
-from .default import DeprovisionHandler
+from azurelinuxagent.common.version import DISTRO_NAME, DISTRO_VERSION, DISTRO_FULL_NAME
 from .arch import ArchDeprovisionHandler
 from .clearlinux import ClearLinuxDeprovisionHandler
 from .coreos import CoreOSDeprovisionHandler
+from .default import DeprovisionHandler
 from .ubuntu import UbuntuDeprovisionHandler, Ubuntu1804DeprovisionHandler
 
 
-from distutils.version import LooseVersion as Version
-
-
 def get_deprovision_handler(distro_name=DISTRO_NAME, 
                             distro_version=DISTRO_VERSION,
                             distro_full_name=DISTRO_FULL_NAME):
@@ -38,10 +35,10 @@ def get_deprovision_handler(distro_name=DISTRO_NAME,
             return Ubuntu1804DeprovisionHandler()
         else:
             return UbuntuDeprovisionHandler()
-    if distro_name == "coreos":
+    if distro_name in ("flatcar", "coreos"):
         return CoreOSDeprovisionHandler()
     if "Clear Linux" in distro_full_name:
-        return ClearLinuxDeprovisionHandler()
+        return ClearLinuxDeprovisionHandler()  # pylint: disable=E1120
 
     return DeprovisionHandler()
 
