@@ -17,39 +17,46 @@
 # Requires Python 2.6+ and Openssl 1.0+
 
 import json
+import os
+import unittest
 
-import azurelinuxagent.common.protocol.imds as imds
+from azurelinuxagent.common.protocol import imds
 
 from azurelinuxagent.common.datacontract import set_properties
 from azurelinuxagent.common.exception import HttpError, ResourceGoneError
 from azurelinuxagent.common.future import ustr, httpclient
 from azurelinuxagent.common.utils import restutil
-from tests.ga.test_update import ResponseMock
-from tests.tools import *
+from tests.protocol.mocks import MockHttpResponse
+from tests.tools import AgentTestCase, data_dir, MagicMock, Mock, patch
+
+
+def get_mock_compute_response():
+    return MockHttpResponse(status=httpclient.OK, body='''{
+    "location": "westcentralus",
+    "name": "unit_test",
+    "offer": "UnitOffer",
+    "osType": "Linux",
+    "placementGroupId": "",
+    "platformFaultDomain": "0",
+    "platformUpdateDomain": "0",
+    "publisher": "UnitPublisher",
+    "resourceGroupName": "UnitResourceGroupName",
+    "sku": "UnitSku",
+    "subscriptionId": "e4402c6c-2804-4a0a-9dee-d61918fc4d28",
+    "tags": "Key1:Value1;Key2:Value2",
+    "vmId": "f62f23fb-69e2-4df0-a20b-cb5c201a3e7a",
+    "version": "UnitVersion",
+    "vmSize": "Standard_D1_v2"
+    }'''.encode('utf-8'))
 
 
 class TestImds(AgentTestCase):
-    @patch("azurelinuxagent.ga.update.restutil.http_get")
-    def test_get(self, mock_http_get):
-        mock_http_get.return_value = ResponseMock(response='''{
-        "location": "westcentralus",
-        "name": "unit_test",
-        "offer": "UnitOffer",
-        "osType": "Linux",
-        "placementGroupId": "",
-        "platformFaultDomain": "0",
-        "platformUpdateDomain": "0",
-        "publisher": "UnitPublisher",
-        "resourceGroupName": "UnitResourceGroupName",
-        "sku": "UnitSku",
-        "subscriptionId": "e4402c6c-2804-4a0a-9dee-d61918fc4d28",
-        "tags": "Key1:Value1;Key2:Value2",
-        "vmId": "f62f23fb-69e2-4df0-a20b-cb5c201a3e7a",
-        "version": "UnitVersion",
-        "vmSize": "Standard_D1_v2"
-        }'''.encode('utf-8'))
 
-        test_subject = imds.ImdsClient()
+    @patch("azurelinuxagent.common.protocol.imds.restutil.http_get")
+    def test_get(self, mock_http_get):
+        mock_http_get.return_value = get_mock_compute_response()
+
+        test_subject = imds.ImdsClient(restutil.KNOWN_WIRESERVER_IP)
         test_subject.get_compute()
 
         self.assertEqual(1, mock_http_get.call_count)
@@ -60,25 +67,25 @@ class TestImds(AgentTestCase):
         self.assertTrue('Metadata' in kw_args['headers'])
         self.assertEqual(True, kw_args['headers']['Metadata'])
 
-    @patch("azurelinuxagent.ga.update.restutil.http_get")
+    @patch("azurelinuxagent.common.protocol.imds.restutil.http_get")
     def test_get_bad_request(self, mock_http_get):
-        mock_http_get.return_value = ResponseMock(status=restutil.httpclient.BAD_REQUEST)
+        mock_http_get.return_value = MockHttpResponse(status=restutil.httpclient.BAD_REQUEST)
 
-        test_subject = imds.ImdsClient()
+        test_subject = imds.ImdsClient(restutil.KNOWN_WIRESERVER_IP)
         self.assertRaises(HttpError, test_subject.get_compute)
 
-    @patch("azurelinuxagent.ga.update.restutil.http_get")
+    @patch("azurelinuxagent.common.protocol.imds.restutil.http_get")
     def test_get_internal_service_error(self, mock_http_get):
-        mock_http_get.return_value = ResponseMock(status=restutil.httpclient.INTERNAL_SERVER_ERROR)
+        mock_http_get.return_value = MockHttpResponse(status=restutil.httpclient.INTERNAL_SERVER_ERROR)
 
-        test_subject = imds.ImdsClient()
+        test_subject = imds.ImdsClient(restutil.KNOWN_WIRESERVER_IP)
         self.assertRaises(HttpError, test_subject.get_compute)
 
-    @patch("azurelinuxagent.ga.update.restutil.http_get")
+    @patch("azurelinuxagent.common.protocol.imds.restutil.http_get")
     def test_get_empty_response(self, mock_http_get):
-        mock_http_get.return_value = ResponseMock(response=''.encode('utf-8'))
+        mock_http_get.return_value = MockHttpResponse(status=httpclient.OK, body=''.encode('utf-8'))
 
-        test_subject = imds.ImdsClient()
+        test_subject = imds.ImdsClient(restutil.KNOWN_WIRESERVER_IP)
         self.assertRaises(ValueError, test_subject.get_compute)
 
     def test_deserialize_ComputeInfo(self):
@@ -102,7 +109,7 @@ class TestImds(AgentTestCase):
         "zone": "In"
         }'''
 
-        data = json.loads(s, encoding='utf-8')
+        data = json.loads(s)
 
         compute_info = imds.ComputeInfo()
         set_properties("compute", compute_info, data)
@@ -251,7 +258,7 @@ class TestImds(AgentTestCase):
             "version": "{3}"
         }}'''.format(publisher, offer, sku, version)
 
-        data = json.loads(s, encoding='utf-8')
+        data = json.loads(s)
         compute_info = imds.ComputeInfo()
         set_properties("compute", compute_info, data)
 
@@ -352,11 +359,11 @@ class TestImds(AgentTestCase):
             return fh.read()
 
     def _assert_validation(self, http_status_code, http_response, expected_valid, expected_response):
-        test_subject = imds.ImdsClient()
+        test_subject = imds.ImdsClient(restutil.KNOWN_WIRESERVER_IP)
         with patch("azurelinuxagent.common.utils.restutil.http_get") as mock_http_get:
-            mock_http_get.return_value = ResponseMock(status=http_status_code,
+            mock_http_get.return_value = MockHttpResponse(status=http_status_code,
                                                       reason='reason',
-                                                      response=http_response)
+                                                      body=http_response)
             validate_response = test_subject.validate()
 
         self.assertEqual(1, mock_http_get.call_count)
@@ -373,15 +380,13 @@ class TestImds(AgentTestCase):
                         "Expected: '{0}', Actual: '{1}'"
                         .format(expected_response, validate_response[1]))
 
-    @patch("azurelinuxagent.common.protocol.util.ProtocolUtil")
-    def test_endpoint_fallback(self, ProtocolUtil):
+    def test_endpoint_fallback(self):
         # http error status codes are tested in test_response_validation, none of which
         # should trigger a fallback. This is confirmed as _assert_validation will count
         # http GET calls and enforces a single GET call (fallback would cause 2) and
         # checks the url called.
 
-        test_subject = imds.ImdsClient()
-        ProtocolUtil().get_wireserver_endpoint.return_value = "foo.bar"
+        test_subject = imds.ImdsClient("foo.bar")
 
         # ensure user-agent gets set correctly
         for is_health, expected_useragent in [(False, restutil.HTTP_USER_AGENT), (True, restutil.HTTP_USER_AGENT_HEALTH)]:
@@ -393,7 +398,7 @@ class TestImds(AgentTestCase):
                 test_subject._http_get = Mock(side_effect=self._mock_http_get)
                 self._mock_imds_setup(primary_ioerror=has_primary_ioerror, secondary_ioerror=True)
                 result = test_subject.get_metadata(resource_path=resource_path, is_health=is_health)
-                self.assertFalse(result.success) if has_primary_ioerror else self.assertTrue(result.success)
+                self.assertFalse(result.success) if has_primary_ioerror else self.assertTrue(result.success)  # pylint: disable=expression-not-assigned
                 self.assertFalse(result.service_error)
                 if has_primary_ioerror:
                     self.assertEqual('IMDS error in /metadata/{0}: Unable to connect to endpoint'.format(resource_path), result.response)
@@ -453,12 +458,12 @@ class TestImds(AgentTestCase):
                 self.assertEqual(2 if has_primary_ioerror else 1, test_subject._http_get.call_count)
 
     def _mock_imds_setup(self, primary_ioerror=False, secondary_ioerror=False, gone_error=False, throttled=False, bad_request=False):
-        self._mock_imds_expect_fallback = primary_ioerror
-        self._mock_imds_primary_ioerror = primary_ioerror
-        self._mock_imds_secondary_ioerror = secondary_ioerror
-        self._mock_imds_gone_error = gone_error
-        self._mock_imds_throttled = throttled
-        self._mock_imds_bad_request = bad_request
+        self._mock_imds_expect_fallback = primary_ioerror  # pylint: disable=attribute-defined-outside-init
+        self._mock_imds_primary_ioerror = primary_ioerror  # pylint: disable=attribute-defined-outside-init
+        self._mock_imds_secondary_ioerror = secondary_ioerror  # pylint: disable=attribute-defined-outside-init
+        self._mock_imds_gone_error = gone_error  # pylint: disable=attribute-defined-outside-init
+        self._mock_imds_throttled = throttled  # pylint: disable=attribute-defined-outside-init
+        self._mock_imds_bad_request = bad_request  # pylint: disable=attribute-defined-outside-init
 
     def _mock_http_get(self, *_, **kwargs):
         if "foo.bar" == kwargs['endpoint'] and not self._mock_imds_expect_fallback:
