@@ -23,24 +23,28 @@ import os
 import pprint
 import re
 import shutil
+import stat
+import sys
 import tempfile
+import time
 import unittest
 from functools import wraps
+from threading import currentThread
 
-import time
-
-import azurelinuxagent.common.event as event
 import azurelinuxagent.common.conf as conf
+import azurelinuxagent.common.event as event
 import azurelinuxagent.common.logger as logger
+from azurelinuxagent.common.future import range  # pylint: disable=redefined-builtin
 from azurelinuxagent.common.utils import fileutil
-
 from azurelinuxagent.common.version import PY_VERSION_MAJOR
 
-# Import mock module for Python2 and Python3
 try:
-    from unittest.mock import Mock, patch, MagicMock, ANY, DEFAULT, call
+    from unittest.mock import Mock, patch, MagicMock, ANY, DEFAULT, call, PropertyMock  # pylint: disable=unused-import
+
+    # Import mock module for Python2 and Python3
+    from bin.waagent2 import Agent  # pylint: disable=unused-import
 except ImportError:
-    from mock import Mock, patch, MagicMock, ANY, DEFAULT, call
+    from mock import Mock, patch, MagicMock, ANY, DEFAULT, call, PropertyMock
 
 test_dir = os.path.dirname(os.path.abspath(__file__))
 data_dir = os.path.join(test_dir, "data")
@@ -54,15 +58,22 @@ if debug:
     logger.add_logger_appender(logger.AppenderType.STDOUT, 
                                logger.LogLevel.VERBOSE)
 
-
-def _do_nothing():
-    pass
-
-
 _MAX_LENGTH = 120
 
 _MAX_LENGTH_SAFE_REPR = 80
 
+# Mock sleep to reduce test execution time
+_SLEEP = time.sleep
+
+
+def mock_sleep(sec=0.01):
+    """
+    Mocks the time.sleep method to reduce unit test time
+    :param sec: Time to replace the sleep call with, default = 0.01sec
+    """
+    _SLEEP(sec)
+
+
 def safe_repr(obj, short=False):
     try:
         result = repr(obj)
@@ -102,23 +113,43 @@ def _safe_repr(obj, short=False):
     return result[:_MAX_LENGTH] + ' [truncated]...'
 
 
+def i_am_root():
+    return os.geteuid() == 0
+
+
+def is_python_version_26():
+    return sys.version_info[0] == 2 and sys.version_info[1] == 6
+
+
+def is_python_version_34():
+    return sys.version_info[0] == 3 and sys.version_info[1] == 4
+
+
+def is_python_version_26_or_34():
+    return is_python_version_26() or is_python_version_34()
+
+
 class AgentTestCase(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         # Setup newer unittest assertions missing in prior versions of Python
 
         if not hasattr(cls, "assertRegex"):
-            cls.assertRegex = cls.assertRegexpMatches if hasattr(cls, "assertRegexpMatches") else _do_nothing
+            cls.assertRegex = cls.assertRegexpMatches if hasattr(cls, "assertRegexpMatches") else cls.emulate_assertRegexpMatches
         if not hasattr(cls, "assertNotRegex"):
-            cls.assertNotRegex = cls.assertNotRegexpMatches if hasattr(cls, "assertNotRegexpMatches") else _do_nothing
+            cls.assertNotRegex = cls.assertNotRegexpMatches if hasattr(cls, "assertNotRegexpMatches") else cls.emulate_assertNotRegexpMatches  # pylint: disable=no-member
         if not hasattr(cls, "assertIn"):
             cls.assertIn = cls.emulate_assertIn
         if not hasattr(cls, "assertNotIn"):
             cls.assertNotIn = cls.emulate_assertNotIn
         if not hasattr(cls, "assertGreater"):
             cls.assertGreater = cls.emulate_assertGreater
+        if not hasattr(cls, "assertGreaterEqual"):
+            cls.assertGreaterEqual = cls.emulate_assertGreaterEqual
         if not hasattr(cls, "assertLess"):
             cls.assertLess = cls.emulate_assertLess
+        if not hasattr(cls, "assertLessEqual"):
+            cls.assertLessEqual = cls.emulate_assertLessEqual
         if not hasattr(cls, "assertIsNone"):
             cls.assertIsNone = cls.emulate_assertIsNone
         if not hasattr(cls, "assertIsNotNone"):
@@ -129,6 +160,18 @@ class AgentTestCase(unittest.TestCase):
             cls.assertRaisesRegex = cls.emulate_raises_regex
         if not hasattr(cls, "assertListEqual"):
             cls.assertListEqual = cls.emulate_assertListEqual
+        if not hasattr(cls, "assertIsInstance"):
+            cls.assertIsInstance = cls.emulate_assertIsInstance
+        if sys.version_info < (2, 7):
+            # assertRaises does not implement a context manager in 2.6; override it with emulate_assertRaises but
+            # keep a pointer to the original implementation to use when a context manager is not requested.
+            cls.original_assertRaises = unittest.TestCase.assertRaises
+            cls.assertRaises = cls.emulate_assertRaises
+            cls.assertDictEqual = cls.emulate_assertDictEqual
+
+    @classmethod
+    def tearDownClass(cls):
+        pass
 
     def setUp(self):
         prefix = "{0}_".format(self.__class__.__name__)
@@ -166,11 +209,21 @@ class AgentTestCase(unittest.TestCase):
             msg = msg if msg is not None else '{0} not greater than {1}'.format(_safe_repr(a), _safe_repr(b))
             self.fail(msg)
 
+    def emulate_assertGreaterEqual(self, a, b, msg=None):
+        if not a >= b:
+            msg = msg if msg is not None else '{0} not greater or equal to {1}'.format(_safe_repr(a), _safe_repr(b))
+            self.fail(msg)
+
     def emulate_assertLess(self, a, b, msg=None):
         if not a < b:
             msg = msg if msg is not None else '{0} not less than {1}'.format(_safe_repr(a), _safe_repr(b))
             self.fail(msg)
 
+    def emulate_assertLessEqual(self, a, b, msg=None):
+        if not a <= b:
+            msg = msg if msg is not None else '{0} not less or equal to {1}'.format(_safe_repr(a), _safe_repr(b))
+            self.fail(msg)
+
     def emulate_assertIsNone(self, x, msg=None):
         if x is not None:
             msg = msg if msg is not None else '{0} is not None'.format(_safe_repr(x))
@@ -181,6 +234,56 @@ class AgentTestCase(unittest.TestCase):
             msg = msg if msg is not None else '{0} is None'.format(_safe_repr(x))
             self.fail(msg)
 
+    def emulate_assertRegexpMatches(self, text, regexp, msg=None):
+        if re.search(regexp, text) is not None:
+            return
+        msg = msg if msg is not None else "'{0}' does not match '{1}'.".format(text, regexp)
+        self.fail(msg)
+
+    def emulate_assertNotRegexpMatches(self, text, regexp, msg=None):
+        if re.search(regexp, text, flags=1) is None:
+            return
+        msg = msg if msg is not None else "'{0}' should not match '{1}'.".format(text, regexp)
+        self.fail(msg)
+
+    class _AssertRaisesContextManager(object):
+        def __init__(self, expected_exception_type, test_case, match_regex=None, regex_flags=0):
+            self._expected_exception_type = expected_exception_type
+            self._test_case = test_case
+            self._match_regex = match_regex
+            self._regex_flags = regex_flags
+            self.exception = None
+
+        def __enter__(self):
+            return self
+
+        @staticmethod
+        def _get_type_name(t):
+            return t.__name__ if hasattr(t, "__name__") else str(t)
+
+        def __exit__(self, exception_type, exception, *_):
+            if exception_type is None:
+                expected = AgentTestCase._AssertRaisesContextManager._get_type_name(self._expected_exception_type)
+                self._test_case.fail("Did not raise an exception; expected '{0}'".format(expected))
+            if not issubclass(exception_type, self._expected_exception_type):
+                raised = AgentTestCase._AssertRaisesContextManager._get_type_name(exception_type)
+                expected = AgentTestCase._AssertRaisesContextManager._get_type_name(self._expected_exception_type)
+                self._test_case.fail("Raised '{0}', but expected '{1}'".format(raised, expected))
+            if self._match_regex is not None:
+                exception_text = str(exception)
+                if re.search(self._match_regex, exception_text, flags=self._regex_flags) is None:
+                    self._test_case.fail("The exception did not match the expected pattern. Expected: r'{0}' Got: '{1}'".format(self._match_regex, exception_text))
+            self.exception = exception
+            return True
+
+    def emulate_assertRaises(self, exception_type, function=None, *args, **kwargs):  # pylint: disable=keyword-arg-before-vararg
+        # return a context manager only when function is not provided; otherwise use the original assertRaises
+        if function is None:
+            return AgentTestCase._AssertRaisesContextManager(exception_type, self)
+
+        self.original_assertRaises(exception_type, function, *args, **kwargs)
+
+        return None
 
     def emulate_raises_regex(self, exception_type, regex, function, *args, **kwargs):
         try:
@@ -193,6 +296,26 @@ class AgentTestCase(unittest.TestCase):
                     exception_type, regex, str(e)))
         self.fail("No exception was thrown.  Expected exception {0} matching {1}".format(exception_type, regex))
 
+    def assertRaisesRegexCM(self, exception_type, regex, flags=0):
+        """
+        Similar to assertRaisesRegex, but returns a context manager (mostly needed for Python 2.*, which does not have a assertRaisesRegex)
+        """
+        return AgentTestCase._AssertRaisesContextManager(exception_type, self, match_regex=regex, regex_flags=flags)
+
+    def emulate_assertDictEqual(self, first, second, msg=None):
+        def fail(message):
+            self.fail(self._formatMessage(msg, message))
+
+        for k in first.keys():
+            if k not in second:
+                fail("'{0}' is missing from second".format(k))
+            if first[k] != second[k]:
+                fail("'{0}' != '{1}' (key: {2})".format(first[k], second[k], k))
+
+        for k in second.keys():
+            if k not in first:
+                fail("'{0}' is missing from first".format(k))
+
     def emulate_assertListEqual(self, seq1, seq2, msg=None, seq_type=None):
         """An equality assertion for ordered sequences (like lists and tuples).
 
@@ -245,7 +368,7 @@ class AgentTestCase(unittest.TestCase):
             elements = (seq_type_name.capitalize(), seq1_repr, seq2_repr)
             differing = '%ss differ: %s != %s\n' % elements
 
-            for i in xrange(min(len1, len2)):
+            for i in range(min(len1, len2)):
                 try:
                     item1 = seq1[i]
                 except (TypeError, IndexError, NotImplementedError):
@@ -296,6 +419,12 @@ class AgentTestCase(unittest.TestCase):
         msg = self._formatMessage(msg, standardMsg)
         self.fail(msg)
 
+    def emulate_assertIsInstance(self, obj, object_type, msg=None):
+        if not isinstance(obj, object_type):
+            msg = msg if msg is not None else '{0} is not an instance of {1}'.format(_safe_repr(obj),
+                                                                                     _safe_repr(object_type))
+            self.fail(msg)
+
     @staticmethod
     def _create_files(tmp_dir, prefix, suffix, count, with_sleep=0):
         for i in range(count):
@@ -303,6 +432,37 @@ class AgentTestCase(unittest.TestCase):
             fileutil.write_file(f, "faux content")
             time.sleep(with_sleep)
 
+    @staticmethod
+    def create_script(script_file, contents):
+        """
+        Creates an executable script with the given contents. If file ends with ".py", it creates a Python3 script,
+        otherwise it creates a bash script.
+        """
+        with open(script_file, "w") as script:
+            if script_file.endswith(".py"):
+                script.write("#!/usr/bin/env python3\n")
+            else:
+                script.write("#!/usr/bin/env bash\n")
+            script.write(contents)
+
+        os.chmod(script_file, stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR)
+
+
+class AgentTestCaseWithGetVmSizeMock(AgentTestCase):
+    
+    def setUp(self):
+        
+        self._get_vm_size_patch = patch('azurelinuxagent.ga.update.UpdateHandler._get_vm_size', return_value="unknown")
+        self._get_vm_size_patch.start()
+        
+        super(AgentTestCaseWithGetVmSizeMock, self).setUp()
+
+    def tearDown(self):
+
+        if self._get_vm_size_patch:
+            self._get_vm_size_patch.stop()
+
+        super(AgentTestCaseWithGetVmSizeMock, self).tearDown()
 
 def load_data(name):
     """Load test data"""
@@ -311,9 +471,11 @@ def load_data(name):
         return data_file.read()
 
 
-def load_bin_data(name):
+def load_bin_data(name, directory=None):
     """Load test bin data"""
-    path = os.path.join(data_dir, name)
+    if directory is None:
+        directory = data_dir
+    path = os.path.join(directory, name)
     with open(path, "rb") as data_file:
         return data_file.read()
 
@@ -326,6 +488,7 @@ supported_distro = [
     ["ubuntu", "15.10", "Snappy Ubuntu Core"],
 
     ["coreos", "", ""],
+    ["flatcar", "", ""],
 
     ["suse", "12", "SUSE Linux Enterprise Server"],
     ["suse", "13.2", "openSUSE"],
@@ -347,6 +510,11 @@ def open_patch():
     return open_name
 
 
+def patch_builtin(target, *args, **kwargs):
+    prefix = 'builtins' if PY_VERSION_MAJOR >= 3 else '__builtin__'
+    return patch("{0}.{1}".format(prefix, target), *args, **kwargs)
+
+
 def distros(distro_name=".*", distro_version=".*", distro_full_name=".*"):
     """Run test on multiple distros"""
     def decorator(test_method):
@@ -371,3 +539,9 @@ def distros(distro_name=".*", distro_version=".*", distro_full_name=".*"):
     return decorator
 
 
+def clear_singleton_instances(cls):
+    # Adding this lock to avoid any race conditions
+    with cls._lock:
+        obj_name = "%s__%s" % (cls.__name__, currentThread().getName())  # Object Name = className__threadName
+        if obj_name in cls._instances:
+            del cls._instances[obj_name]
