@@ -15,11 +15,14 @@
 # Requires Python 2.6+ and Openssl 1.0+
 #
 
+import os
+import unittest
+
 from azurelinuxagent.common.exception import HttpError, ResourceGoneError, InvalidContainerError
 import azurelinuxagent.common.utils.restutil as restutil
 from azurelinuxagent.common.utils.restutil import HTTP_USER_AGENT
 from azurelinuxagent.common.future import httpclient, ustr
-from tests.tools import *
+from tests.tools import AgentTestCase, call, Mock, MagicMock, patch
 
 
 class TestIOErrorCounter(AgentTestCase):
@@ -28,7 +31,7 @@ class TestIOErrorCounter(AgentTestCase):
         restutil.IOErrorCounter.set_protocol_endpoint()
 
         restutil.IOErrorCounter.increment(
-            restutil.DEFAULT_PROTOCOL_ENDPOINT, restutil.HOST_PLUGIN_PORT)
+            restutil.KNOWN_WIRESERVER_IP, restutil.HOST_PLUGIN_PORT)
 
         counts = restutil.IOErrorCounter.get_and_reset()
         self.assertEqual(1, counts["hostplugin"])
@@ -40,7 +43,7 @@ class TestIOErrorCounter(AgentTestCase):
         restutil.IOErrorCounter.set_protocol_endpoint()
 
         restutil.IOErrorCounter.increment(
-            restutil.DEFAULT_PROTOCOL_ENDPOINT, 80)
+            restutil.KNOWN_WIRESERVER_IP, 80)
 
         counts = restutil.IOErrorCounter.get_and_reset()
         self.assertEqual(0, counts["hostplugin"])
@@ -64,11 +67,11 @@ class TestIOErrorCounter(AgentTestCase):
         restutil.IOErrorCounter.set_protocol_endpoint()
 
         restutil.IOErrorCounter.increment(
-            restutil.DEFAULT_PROTOCOL_ENDPOINT, restutil.HOST_PLUGIN_PORT)
+            restutil.KNOWN_WIRESERVER_IP, restutil.HOST_PLUGIN_PORT)
         restutil.IOErrorCounter.increment(
-            restutil.DEFAULT_PROTOCOL_ENDPOINT, restutil.HOST_PLUGIN_PORT)
+            restutil.KNOWN_WIRESERVER_IP, restutil.HOST_PLUGIN_PORT)
         restutil.IOErrorCounter.increment(
-            restutil.DEFAULT_PROTOCOL_ENDPOINT, 80)
+            restutil.KNOWN_WIRESERVER_IP, 80)
         restutil.IOErrorCounter.increment(
             '169.254.169.254', 80)
         restutil.IOErrorCounter.increment(
@@ -86,48 +89,67 @@ class TestIOErrorCounter(AgentTestCase):
 class TestHttpOperations(AgentTestCase):
     def test_parse_url(self):
         test_uri = "http://abc.def/ghi#hash?jkl=mn"
-        host, port, secure, rel_uri = restutil._parse_url(test_uri)
-        self.assertEquals("abc.def", host)
-        self.assertEquals("/ghi#hash?jkl=mn", rel_uri)
+        host, port, secure, rel_uri = restutil._parse_url(test_uri)  # pylint: disable=unused-variable
+        self.assertEqual("abc.def", host) 
+        self.assertEqual("/ghi#hash?jkl=mn", rel_uri) 
 
         test_uri = "http://abc.def/"
         host, port, secure, rel_uri = restutil._parse_url(test_uri)
-        self.assertEquals("abc.def", host)
-        self.assertEquals("/", rel_uri)
-        self.assertEquals(False, secure)
+        self.assertEqual("abc.def", host) 
+        self.assertEqual("/", rel_uri) 
+        self.assertEqual(False, secure) 
 
         test_uri = "https://abc.def/ghi?jkl=mn"
         host, port, secure, rel_uri = restutil._parse_url(test_uri)
-        self.assertEquals(True, secure)
+        self.assertEqual(True, secure) 
 
         test_uri = "http://abc.def:80/"
         host, port, secure, rel_uri = restutil._parse_url(test_uri)
-        self.assertEquals("abc.def", host)
+        self.assertEqual("abc.def", host) 
 
         host, port, secure, rel_uri = restutil._parse_url("")
-        self.assertEquals(None, host)
-        self.assertEquals(rel_uri, "")
+        self.assertEqual(None, host) 
+        self.assertEqual(rel_uri, "") 
 
         host, port, secure, rel_uri = restutil._parse_url("None")
-        self.assertEquals(None, host)
-        self.assertEquals(rel_uri, "None")
+        self.assertEqual(None, host) 
+        self.assertEqual(rel_uri, "None") 
+
+    def test_trim_url_parameters(self):
+        test_uri = "http://abc.def/ghi#hash?jkl=mn"
+        rel_uri = restutil._trim_url_parameters(test_uri)
+        self.assertEqual("http://abc.def/ghi", rel_uri)
+
+        test_uri = "https://abc.def/ghi?jkl=mn"
+        rel_uri = restutil._trim_url_parameters(test_uri)
+        self.assertEqual("https://abc.def/ghi", rel_uri)
+
+        test_uri = "https://abc.def:8443/ghi?jkl=mn"
+        rel_uri = restutil._trim_url_parameters(test_uri)
+        self.assertEqual("https://abc.def:8443/ghi", rel_uri) 
+
+        rel_uri = restutil._trim_url_parameters("")
+        self.assertEqual(rel_uri, "")
+
+        rel_uri = restutil._trim_url_parameters("None")
+        self.assertEqual(rel_uri, "None")
 
     def test_cleanup_sas_tokens_from_urls_for_normal_cases(self):
         test_url = "http://abc.def/ghi#hash?jkl=mn"
         filtered_url = restutil.redact_sas_tokens_in_urls(test_url)
-        self.assertEquals(test_url, filtered_url)
+        self.assertEqual(test_url, filtered_url) 
 
         test_url = "http://abc.def:80/"
         filtered_url = restutil.redact_sas_tokens_in_urls(test_url)
-        self.assertEquals(test_url, filtered_url)
+        self.assertEqual(test_url, filtered_url) 
 
         test_url = "http://abc.def/"
         filtered_url = restutil.redact_sas_tokens_in_urls(test_url)
-        self.assertEquals(test_url, filtered_url)
+        self.assertEqual(test_url, filtered_url) 
 
         test_url = "https://abc.def/ghi?jkl=mn"
         filtered_url = restutil.redact_sas_tokens_in_urls(test_url)
-        self.assertEquals(test_url, filtered_url)
+        self.assertEqual(test_url, filtered_url) 
 
     def test_cleanup_sas_tokens_from_urls_containing_sas_tokens(self):
         # Contains pair of URLs (RawURL, RedactedURL)
@@ -172,7 +194,7 @@ class TestHttpOperations(AgentTestCase):
                        ]
 
         for x in urls_tuples:
-            self.assertEquals(restutil.redact_sas_tokens_in_urls(x[0]), x[1])
+            self.assertEqual(restutil.redact_sas_tokens_in_urls(x[0]), x[1]) 
 
     @patch('azurelinuxagent.common.conf.get_httpproxy_port')
     @patch('azurelinuxagent.common.conf.get_httpproxy_host')
@@ -244,7 +266,7 @@ class TestHttpOperations(AgentTestCase):
         }):
             no_proxy_from_environment = restutil.get_no_proxy()
 
-            self.assertEquals(len(no_proxy_list), len(no_proxy_from_environment))
+            self.assertEqual(len(no_proxy_list), len(no_proxy_from_environment)) 
 
             for i, j in zip(no_proxy_from_environment, no_proxy_list):
                 self.assertEqual(i, j)
@@ -258,7 +280,7 @@ class TestHttpOperations(AgentTestCase):
         }):
             no_proxy_from_environment = restutil.get_no_proxy()
 
-            self.assertEquals(len(no_proxy_list_cleaned), len(no_proxy_from_environment))
+            self.assertEqual(len(no_proxy_list_cleaned), len(no_proxy_from_environment)) 
 
             for i, j in zip(no_proxy_from_environment, no_proxy_list_cleaned):
                 print(i, j)
@@ -274,7 +296,7 @@ class TestHttpOperations(AgentTestCase):
         }):
             no_proxy_from_environment = restutil.get_no_proxy()
 
-            self.assertEquals(len(no_proxy_list), len(no_proxy_from_environment))
+            self.assertEqual(len(no_proxy_list), len(no_proxy_from_environment)) 
 
             for i, j in zip(no_proxy_from_environment, no_proxy_list):
                 self.assertEqual(i, j)
@@ -301,11 +323,11 @@ class TestHttpOperations(AgentTestCase):
         self.assertFalse(restutil.address_in_network('172.16.0.1', '192.168.1.0/24'))
 
     def test_dotted_netmask(self):
-        self.assertEquals(restutil.dotted_netmask(0), '0.0.0.0')
-        self.assertEquals(restutil.dotted_netmask(8), '255.0.0.0')
-        self.assertEquals(restutil.dotted_netmask(16), '255.255.0.0')
-        self.assertEquals(restutil.dotted_netmask(24), '255.255.255.0')
-        self.assertEquals(restutil.dotted_netmask(32), '255.255.255.255')
+        self.assertEqual(restutil.dotted_netmask(0), '0.0.0.0') 
+        self.assertEqual(restutil.dotted_netmask(8), '255.0.0.0') 
+        self.assertEqual(restutil.dotted_netmask(16), '255.255.0.0') 
+        self.assertEqual(restutil.dotted_netmask(24), '255.255.255.0') 
+        self.assertEqual(restutil.dotted_netmask(32), '255.255.255.255') 
         self.assertRaises(ValueError, restutil.dotted_netmask, 33)
 
     def test_bypass_proxy(self):
@@ -330,7 +352,7 @@ class TestHttpOperations(AgentTestCase):
 
         HTTPConnection.return_value = mock_conn
 
-        resp = restutil._http_request("GET", "foo", "/bar")
+        resp = restutil._http_request("GET", "foo", "/bar", 10)
 
         HTTPConnection.assert_has_calls([
             call("foo", 80, timeout=10)
@@ -340,8 +362,8 @@ class TestHttpOperations(AgentTestCase):
             call(method="GET", url="/bar", body=None, headers={'User-Agent': HTTP_USER_AGENT, 'Connection': 'close'})
         ])
         self.assertEqual(1, mock_conn.getresponse.call_count)
-        self.assertNotEquals(None, resp)
-        self.assertEquals("TheResults", resp.read())
+        self.assertNotEqual(None, resp) 
+        self.assertEqual("TheResults", resp.read()) 
 
     @patch("azurelinuxagent.common.future.httpclient.HTTPSConnection")
     @patch("azurelinuxagent.common.future.httpclient.HTTPConnection")
@@ -353,7 +375,7 @@ class TestHttpOperations(AgentTestCase):
 
         HTTPSConnection.return_value = mock_conn
 
-        resp = restutil._http_request("GET", "foo", "/bar", secure=True)
+        resp = restutil._http_request("GET", "foo", "/bar", 10, secure=True)
 
         HTTPConnection.assert_not_called()
         HTTPSConnection.assert_has_calls([
@@ -363,8 +385,8 @@ class TestHttpOperations(AgentTestCase):
             call(method="GET", url="/bar", body=None, headers={'User-Agent': HTTP_USER_AGENT, 'Connection': 'close'})
         ])
         self.assertEqual(1, mock_conn.getresponse.call_count)
-        self.assertNotEquals(None, resp)
-        self.assertEquals("TheResults", resp.read())
+        self.assertNotEqual(None, resp) 
+        self.assertEqual("TheResults", resp.read()) 
 
     @patch("azurelinuxagent.common.future.httpclient.HTTPSConnection")
     @patch("azurelinuxagent.common.future.httpclient.HTTPConnection")
@@ -376,7 +398,7 @@ class TestHttpOperations(AgentTestCase):
 
         HTTPConnection.return_value = mock_conn
 
-        resp = restutil._http_request("GET", "foo", "/bar",
+        resp = restutil._http_request("GET", "foo", "/bar", 10,
                             proxy_host="foo.bar", proxy_port=23333)
 
         HTTPConnection.assert_has_calls([
@@ -387,13 +409,13 @@ class TestHttpOperations(AgentTestCase):
             call(method="GET", url="http://foo:80/bar", body=None, headers={'User-Agent': HTTP_USER_AGENT, 'Connection': 'close'})
         ])
         self.assertEqual(1, mock_conn.getresponse.call_count)
-        self.assertNotEquals(None, resp)
-        self.assertEquals("TheResults", resp.read())
+        self.assertNotEqual(None, resp) 
+        self.assertEqual("TheResults", resp.read()) 
 
     @patch("azurelinuxagent.common.utils.restutil._get_http_proxy")
     @patch("time.sleep")
     @patch("azurelinuxagent.common.utils.restutil._http_request")
-    def test_http_request_proxy_with_no_proxy_check(self, _http_request, sleep, mock_get_http_proxy):
+    def test_http_request_proxy_with_no_proxy_check(self, _http_request, sleep, mock_get_http_proxy):  # pylint: disable=unused-argument
         mock_http_resp = MagicMock()
         mock_http_resp.read = Mock(return_value="hehe")
         _http_request.return_value = mock_http_resp
@@ -405,13 +427,13 @@ class TestHttpOperations(AgentTestCase):
         }):
             # Test http get
             resp = restutil.http_get("http://foo.com", use_proxy=True)
-            self.assertEquals("hehe", resp.read())
-            self.assertEquals(0, mock_get_http_proxy.call_count)
+            self.assertEqual("hehe", resp.read()) 
+            self.assertEqual(0, mock_get_http_proxy.call_count) 
 
             # Test http get
             resp = restutil.http_get("http://bar.com", use_proxy=True)
-            self.assertEquals("hehe", resp.read())
-            self.assertEquals(1, mock_get_http_proxy.call_count)
+            self.assertEqual("hehe", resp.read()) 
+            self.assertEqual(1, mock_get_http_proxy.call_count) 
 
     def test_proxy_conditions_with_no_proxy(self):
         should_use_proxy = True
@@ -423,32 +445,32 @@ class TestHttpOperations(AgentTestCase):
             'no_proxy': ",".join(no_proxy_list)
         }):
             host = "10.0.0.1"
-            self.assertEquals(should_use_proxy, use_proxy and not restutil.bypass_proxy(host))
+            self.assertEqual(should_use_proxy, use_proxy and not restutil.bypass_proxy(host)) 
 
             host = "foo.com"
-            self.assertEquals(should_not_use_proxy, use_proxy and not restutil.bypass_proxy(host))
+            self.assertEqual(should_not_use_proxy, use_proxy and not restutil.bypass_proxy(host)) 
 
             host = "www.google.com"
-            self.assertEquals(should_not_use_proxy, use_proxy and not restutil.bypass_proxy(host))
+            self.assertEqual(should_not_use_proxy, use_proxy and not restutil.bypass_proxy(host)) 
 
             host = "168.63.129.16"
-            self.assertEquals(should_not_use_proxy, use_proxy and not restutil.bypass_proxy(host))
+            self.assertEqual(should_not_use_proxy, use_proxy and not restutil.bypass_proxy(host)) 
 
             host = "www.bar.com"
-            self.assertEquals(should_use_proxy, use_proxy and not restutil.bypass_proxy(host))
+            self.assertEqual(should_use_proxy, use_proxy and not restutil.bypass_proxy(host)) 
 
         no_proxy_list = ["10.0.0.1/24"]
         with patch.dict(os.environ, {
             'no_proxy': ",".join(no_proxy_list)
         }):
             host = "www.bar.com"
-            self.assertEquals(should_use_proxy, use_proxy and not restutil.bypass_proxy(host))
+            self.assertEqual(should_use_proxy, use_proxy and not restutil.bypass_proxy(host)) 
 
             host = "10.0.0.1"
-            self.assertEquals(should_not_use_proxy, use_proxy and not restutil.bypass_proxy(host))
+            self.assertEqual(should_not_use_proxy, use_proxy and not restutil.bypass_proxy(host)) 
 
             host = "10.0.1.1"
-            self.assertEquals(should_use_proxy, use_proxy and not restutil.bypass_proxy(host))
+            self.assertEqual(should_use_proxy, use_proxy and not restutil.bypass_proxy(host)) 
 
         # When No_proxy is empty
         with patch.dict(os.environ, {
@@ -508,7 +530,7 @@ class TestHttpOperations(AgentTestCase):
 
         HTTPSConnection.return_value = mock_conn
 
-        resp = restutil._http_request("GET", "foo", "/bar",
+        resp = restutil._http_request("GET", "foo", "/bar", 10,
                             proxy_host="foo.bar", proxy_port=23333,
                             secure=True)
 
@@ -520,23 +542,23 @@ class TestHttpOperations(AgentTestCase):
             call(method="GET", url="https://foo:443/bar", body=None, headers={'User-Agent': HTTP_USER_AGENT, 'Connection': 'close'})
         ])
         self.assertEqual(1, mock_conn.getresponse.call_count)
-        self.assertNotEquals(None, resp)
-        self.assertEquals("TheResults", resp.read())
+        self.assertNotEqual(None, resp) 
+        self.assertEqual("TheResults", resp.read()) 
 
     @patch("time.sleep")
     @patch("azurelinuxagent.common.utils.restutil._http_request")
-    def test_http_request_with_retry(self, _http_request, sleep):
+    def test_http_request_with_retry(self, _http_request, sleep):  # pylint: disable=unused-argument
         mock_http_resp = MagicMock()
         mock_http_resp.read = Mock(return_value="hehe")
         _http_request.return_value = mock_http_resp
 
         # Test http get
         resp = restutil.http_get("http://foo.bar")
-        self.assertEquals("hehe", resp.read())
+        self.assertEqual("hehe", resp.read()) 
 
         # Test https get
         resp = restutil.http_get("https://foo.bar")
-        self.assertEquals("hehe", resp.read())
+        self.assertEqual("hehe", resp.read()) 
 
         # Test http failure
         _http_request.side_effect = httpclient.HTTPException("Http failure")
@@ -605,7 +627,7 @@ class TestHttpOperations(AgentTestCase):
 
         _http_request.side_effect = [
                 Mock(status=httpclient.SERVICE_UNAVAILABLE)
-                    for i in range(restutil.DEFAULT_RETRIES)
+                    for i in range(restutil.DEFAULT_RETRIES)  # pylint: disable=unused-variable
             ] + [Mock(status=httpclient.OK)]
 
         restutil.http_get("https://foo.bar",
@@ -625,7 +647,7 @@ class TestHttpOperations(AgentTestCase):
 
         _http_request.side_effect = [
                 Mock(status=httpclient.SERVICE_UNAVAILABLE)
-                    for i in range(restutil.THROTTLE_RETRIES-1)
+                    for i in range(restutil.THROTTLE_RETRIES-1)  # pylint: disable=unused-variable
             ] + [Mock(status=httpclient.OK)]
 
         restutil.http_get("https://foo.bar",
